
# q-expansion of newform 450.4.a.j, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 450
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-2], [0], [0], [26], [28], [12], [64], [-60], [58], [-90], [-128], [236], [-242], [362], [-226], [108], [20], [542], [-434], [1128], [632], [-720], [478], [490], [1456], [578], [1462], [-966], [370], [528], [-1534], [-12], [1224], [3100], [-250], [2152], [-524], [-3518], [534], [-4252], [-2500], [-2578], [768], [-2608], [-5116], [-3480], [3132], [62], [5314], [-190], [2408], [5680], [-278], [-3252], [-1536], [4858], [-2610], [-5168], [-1924], [-3042], [-1718], [-2292], [5406], [5688], [7352], [3484], [-7868], [656], [5754], [-3110], [7808], [9240], [-3214], [-348], [4940], [-6142], [-3050], [5396], [-14482], [-1090], [7180], [-8138], [208], [12992], [1080], [9078], [-14310], [-2344], [-11382], [16062], [-17166], [-7520], [-11814], [-14052], [7620], [1818], [-17850], [19238], [-6278], [-9818], [-12514], [-10596], [-14002], [7330], [5812], [16736], [7434], [-25872], [3720], [-12958], [-7214], [-4828], [-27656], [-21220], [17672], [-7322], [-8238], [-6426], [5908], [26780], [-24538], [-28848], [26884], [-14282], [-3428], [-26942], [-1950], [-12080], [17226], [-788], [-2060], [3258], [-4528], [18236], [18678], [27390], [-9252], [5726], [-27236], [-10950], [-8828], [16058], [41862], [12154], [-15390], [4280], [14452], [22584], [-26740], [498], [-13244], [-40842], [-12078], [18294], [22566], [6768], [22200], [6330], [-19544], [9898], [-41406], [-25432], [12106], [-7812], [-12576], [-4342], [26272], [44796], [24110], [54348], [-45900], [53382], [-11192], [35032], [51280], [-25590], [-16108], [46338], [-51658], [1010], [16226], [-56332], [-56828], [3864], [-60382], [34830], [-59404], [51442], [-50290], [40448], [-44448], [25798], [13372], [-45342], [-63726], [-24312], [12042], [24252], [-31296], [48458], [54390], [-26448], [396], [11710], [-40220], [-4596], [-46560], [-32882], [37850], [30532], [-16544], [6138], [-78218], [-7686], [-15880], [-20558], [-18734], [-23122], [70694], [-14972], [26262], [-60680], [-44190], [65422], [26274], [-68070], [27928], [15680], [-20054], [29788], [-28148], [-61820], [-26368], [3918], [23882], [-43346], [112950], [-45652], [-34380], [48168], [98718], [-94348], [-3898], [20170], [-5552], [97320], [8226], [-15372], [104540], [48898], [-93084], [-88802], [84614], [13390], [-28252], [-58500], [-13298], [52966], [117524], [64936], [66222], [-106046], [83170], [-63188], [75504], [-83980], [-101610], [82198], [80522], [-28212], [15542], [-59954], [24712], [43840], [119216], [135862], [14954], [-91270], [105842], [-135132], [110418], [48632], [-50746], [-79778], [-22634], [60688], [-18528], [34484], [108920], [128410], [104338], [153954], [132408], [-61572], [83612], [135870], [89952], [7548], [25300], [-22914], [116872], [61884], [19280], [-12162], [2132], [-112624], [-117286], [25130], [-138040], [-124428], [-166542], [120270], [71838], [-163518], [-28106], [-44890], [95900], [-2272], [-34048], [-21270], [-68548], [117736], [152898], [-64498], [-42312], [-39958], [81300], [-84438], [114094], [-72932], [-24338], [52684], [148160], [47438], [-145788], [136554], [-46430], [-161472], [136402], [-81214], [-85308], [-159976], [-203210], [-26248], [41908], [196620], [185142], [127126], [132848], [-57036], [-208468], [-150984], [-92142], [108462], [-199630], [-98952], [96960], [-213732], [-93056], [-54422], [68636], [-194682], [-1266], [-60140], [95886], [178232], [-84756], [111942], [44802], [-25612], [-175300], [135778], [-66810], [52872], [-46044], [-81860], [16928], [-89568], [-195990], [-202904], [94458], [-25192], [-66374], [-16096], [-70860], [263178], [167952], [214876], [67242], [-198866], [63110], [133068], [-100380], [180526], [-105352], [-101608], [133280], [-132550], [-106108], [10458], [-61790], [-154352], [102226], [206424], [-229770], [50552], [262356], [159438], [181762], [30540], [-125648], [152964], [-110922], [193132], [5576], [-17942], [184480], [-25014], [-137216], [222858], [138270], [86716], [266494], [171540], [26152], [248124], [72638], [-172510], [158292], [281320]]
